/****************************************************************************/
/*                                                                          */
/*    Init.c      :Initialization routines for MaxTime                      */
/*                                                                          */
/****************************************************************************/

#include "MaxTime.h"
#include "globals.h"

char dummy[81];

void init()
{
   int x;
   char drive[_MAX_DRIVE];
   char dir[_MAX_DIR];

   LogFile[0] = 0;		/* Turn Off logfile by default */
   BullFile[0] = 0;
   NewHelpFil[0] = 0;
   TxtFile[0] = 0;
   MaxTimeSession = 30;
   MaxTimeDaily = 60;
   MaxDepoDaily = 60;
   MaxBankSession = 30;

   TextAttr[STD_TEXT] = 6;   		/* Cyan on black */
   TextAttr[HILITE_TEXT] = 3;		/* Yellow on black */
   TextAttr[MENU_KEY] = 2;		/* Green on black */
   TextAttr[TIME_LEFT] = 87;		/* White on Magenta */
   TextAttr[PROMPT_TEXT] = 4;		/* Blue on black */
   TextAttr[ATTN_TEXT] = 1;		/* Red on black */

   load_cfgfile(MCfgFile);

   if (Task && LogFile[0]) {		/* We have to munge the log filename */
      _splitpath(LogFile,drive,dir,temp,temp1);
      temp[6] = 0;		/* Trim out 2 chars */
      sprintf(LogFile,"%s%s%s%02d%s",drive,dir,temp,Task,temp1);
   }

   sprintf(temp,"MxTmAct.%03d",Task);		/* Create a task id */
   x = sopen(temp,O_RDWR | O_BINARY | O_CREAT,SH_DENYWR,S_IWRITE);
   close(x);		/* Create just a null file */
}

/* Close up MaxTime, clean up loose ends */
void deinit(int errcode)
{
   int x,y;

   sprintf(temp,"Restar%02d.bbs",Task);
   if(stat(temp,&fbuf) == 0  && MinutesWanted != 0) { 
      x = sopen(temp,O_BINARY | O_RDWR,SH_DENYWR,S_IWRITE);
      lseek(x,0L,SEEK_SET);		/* Rewind the file */
      if (x != -1) {
         y = timeon() + MinutesWanted;
         if (y >= 0) {		/* We are subtracting time from timebank */
            if ((unsigned long)(y + BBStime) > Rstart->max_time) /* Time is greater than max */
            /* So bump up as much as we can */
               y  -= (int) ((unsigned long)(y + BBStime)- Rstart->max_time);
            Rstart->timeoff += 60 * y;
            Rstart->timeon  += 60 * y;
         }
         else {		/* We are adding time to timebank */
            Rstart->timeon -= 60 * (y * -1);		/* Subtract from our time on */
            Rstart->timeoff -= 60 * (y * -1);		/* Subtract from our time on */
            LastUser.time += (y * -1);
            LastUser.timeremaining -= (y * -1);
         }
         write(x,(struct _restart *) Rstart,sizeof(struct _restart)); 
         close(x);
         if (y < 0) {		/* Subtracting from user online time */
            x = sopen(LastUse,O_RDWR | O_BINARY,SH_DENYWR,S_IWRITE);
            if (x != -1) {
               lseek(x,0L,SEEK_SET);
               write(x,&LastUser,user_slen);
               close(x);
            }
         }
      }
   }

   if (!SysopMode) {
      x = sopen(CfgFile,O_RDWR | O_BINARY,SH_DENYWR,S_IWRITE);
      if (x != -1) {
         if (USERCFG.banktime <= 0)
            USERCFG.banktime = 0;
         time(&USERCFG.lasttime);
         USERCFG.isused = TRUE;
         USERCFG.calls++;
         update_usercfg(x);
         close(x);
      }
      else logit("Error updating User Config file",'!');
      sprintf(temp,"%s off-line. Calls=%u, Len=%u",LastUser.name,USERCFG.calls,timeon());
      logit(temp,':');
   }

   sprintf(temp,"End, (%d)",errcode);
   logit(temp,':');

   if (LogFile[0]) {
      x = sopen(LogFile,O_BINARY | O_RDWR | O_APPEND,SH_DENYWR,S_IWRITE);
      if (x > 0) {
         write(x,"\r\n",2);
         close(x);
      }
   }
   sprintf(temp,"MxTmAct.%03d",Task);		/* Create a task id */
   unlink(temp);

   exit(errcode);
}


void aborterror(int code,char *strng)
{
   int exitcode;

   exitcode = 1;


   switch (code) {
      case BADALLOC:
         strcpy(temp,"Error allocating memory");
         break;

      case BADEXEC:
         strcpy(temp,"Error executing external program");
         break;

      case FILEOPEN:
         strcpy(temp,"Error opening file");
         exitcode = 4;
         break;

      case FILECREATE:
         strcpy(temp,"Error creating file");
         exitcode = 4;
         break;

      case FILEWRITE:
         strcpy(temp,"Error writing file");
         exitcode = 4;
         break;

      case DROPCARRIER:
         strcpy(temp,"User dropped carrier!");
         IsLocal = TRUE;
         exitcode = 3;
         break;

      case USERIDLE:
         sprintf(temp,"User was idle more than %d minutes!",IDLETIME);
         IsLocal = TRUE;
         exitcode = 3;
         break;

      default:
         strcpy(temp,"General failure");
         break;
   }

   setcolor(TextAttr[ATTN_TEXT]);
   strout("\r\n");
   if (strng)  {
      strout(strng);
      logit(strng,'!');
   }
   else {
      strout(temp);
      logit(temp,'!');
   }
   strout("\r\n");
   setcolor(TextAttr[STD_TEXT]);
   deinit(exitcode);
}

int load_cfgfile(char *cfgfile)
{
   FILE *cfile;
   char *p1,*p2;
   int x,y;

   cfile = _fsopen(cfgfile,"rt",SH_DENYNO);
   if (cfile == NULL) {		/* Try to open it in executable maxmail dir */
      _splitpath(cfgfile,dummy,dummy,temp,temp1);		/* Extract just filename */
      _makepath(dummy,hdrive,hdir,temp,temp1);
      cfile = _fsopen(dummy,"rt",SH_DENYNO);
      if (cfile == NULL) {		/* Not there? Then its hopeless! */
         sprintf(temp1,"Error opening '%s' config file!\r\n",dummy);
         aborterror(FILEOPEN,temp1);
      }
   }

   fgets(temp,81,cfile);
   while (!feof(cfile)) {
      if (*temp != '\0') {		/* Parse out token string */
         p1 = strtok(temp," \n\t");
         p2 = strtok(NULL," \n\t;");
         if (strcmpi(p1,";") == 0) {		/* Just ignore comment lines */
            fgets(temp,81,cfile);
            continue;
         }
         if (strcmpi(p1,"TxtFile") == 0) {
            if (p2) strcpy(TxtFile,p2);
         }
         else if (strcmpi(p1,"CfgFile") == 0) {
            if (p2) strcpy(CfgFile,p2);
         }
         else if (strcmpi(p1,"LogFile") == 0) {
            if (p2) strcpy(LogFile,p2);
         }
         else if (strcmpi(p1,"NewHelpFile") == 0) {
            if (p2) strcpy(NewHelpFil,p2);
         }
         else if(strcmpi(p1,"Colors") == 0) {
            strcpy(temp1,p2);		/* Copy colors numbers */
            p2 = strtok(temp1,", ");
            for (x=0; x < 6; x++) {
               if (p2) TextAttr[x] = (byte) atoi(p2);
               else {		/* Sysop didn't give full set! */
                  y = TextAttr[x-1];		/* Fetch last attribute */
                  while (x < 6) 
                     TextAttr[x++] = (byte) y; 	/* And store to remainder */
                  break;
               }
               p2 = strtok(NULL,", ");		/* Fetch next token */
            }
         }
         else if (strcmpi(p1,"Bulletin") == 0) {
            if (p2) strcpy(BullFile,p2);
         }
         else if (strcmpi(p1,"MaxTimeSession") == 0) {
            if (atoi(p2) >= 2)
               MaxTimeSession = (unsigned int) atoi(p2);
         }
         else if (strcmpi(p1,"MaxTimeDaily") == 0) {
            if (atoi(p2) >= 2)
               MaxTimeDaily = (unsigned int) atoi(p2);
         }
         else if (strcmpi(p1,"MaxBankSession") == 0) {
            if (atoi(p2) >= 2)
               MaxBankSession = (unsigned int) atoi(p2);
         }
         else if (strcmpi(p1,"MaxDepositDaily") == 0) {
            if (atoi(p2) >= 2)
               MaxDepoDaily = (unsigned int) atoi(p2);
         }
      }
      fgets(temp,81,cfile);
   }
   fclose(cfile);
   return(TRUE);
}


